#
#
#   This is an example on how to build 
#   a webservice linked to a database
#
#
#
# A typical SQL command is based on command, parameters:
# select name, firstname from address where address.id= :identifier 

# Example of a query
# for multiple parameters:
# MyQuery = """select rna, comment from table where locusCode = %s and id = %s"""
# for more details on how to operate with the Python DB api see: http://www.python.org/topics/database

MyQuery = """select rna, comment from table where locusCode = :locusName"""

def _toSQLParameters(queryData):
    """ Type of transformation:
        [MobyObject, MobyObject, Parameter, Parameter] --> {'paramName1':sqlParameter1, 'paramName2':sqlParameter2, ...}
        an array of moby objects are transformed into a tuple of parameters

        ex, you want to retrieve from a database a RNA sequence given its AGI locusCode:
            the script receive a moby object composed of a namespace and an identifier
             <Object namespace="AGI_LocusCode" id="At3g23130"/>

             toSQLParameters will extract the identifier and put it in a tuple
    """

    identifier = queryData[0].id


    """ Of course you can add more parameters and do all kind of checking

    """
    return {'locusName':identifier}

def _formatter(results):
    """ Type of transformation:
        results given by a Python DB fetchall command --> [] a set of MobyObjects or Collections
        [(col1, col2, col3, etc..), etc..]            --> [("collectionName", [MobyObject ...]), etc..]  or
                                                          [MobyObject1, MobyObject2, etc..]
    """

    #Please note that results MUST be sent inside an array of bioMoby Objects (Collection or bioMoby ontology objects)

    from bioMoby import MobyCommentedRNASequence

    mobyResults = []

    for rowset in results:
        #in this example, we will return a commented RNA sequence
        sequence = rowset[0]
        comment = rowset[1]

        crna = MobyCommentedRNASequence(Length=len(sequence), SequenceString=sequence, Description=comment)
        mobyResults.append(crna)

    #return the bioMoby results to the dispatcher
    return mobyResults

def getCommentedRNAFromAGI(mobyContent):
    """ This is the entry point of your webservice
        it receives a string that contains a MobyContent object and shall return a string containing a 
        MobyContent object

        Hopefully for us! we just use the SQLDispatcher who will do all the painfull job
    """

    #(Import your driver, in this example it is an oracle driver but all Python DB api 2 drivers are accepted)
    import cx_Oracle
    from bioMoby.webservice import SQLDispatcher

    #each driver has its own syntax, this one is the cx_oracle
    dbConnection=cx_Oracle.connect(USERNAME, PASSWORD, cx_Oracle.makedsn(HOST, PORT, SID))

    #instanciate the SQLDispatcher
    d=SQLDispatcher(mobyContent, dbConnection, MyQuery, _toSQLParameters, _formatter)

    #return the results!
    return d.execute()

    """ You don't need to manage the closing of the dbConnection,
        (but in general, Python call the __del__ method to close db connections)
        SQLDispatcher do it for you, formatters and parameters transformation are also managed 
        by the dispatcher. Exceptions are returned in the serviceNotes block
    """


from ZSI import dispatch

#Important: typesmodule=TCBioMoby must be added because of issue with bioMoby Perl
#If this line is dropped, then, your service won't be able to communicate with bioMoby Perl 
from bioMoby.webservice import TCBioMoby
dispatch.AsCGI(typesmodule=TCBioMoby)
